/*
 * foo_tta.h
 *
 * Description:	 TTAv1 plug-in definitions and prototypes
 * Developed by: Alexander Djourik <sasha@iszf.irk.ru>
 *               Pavel Zhilin <pzh@iszf.irk.ru>
 *
 * Copyright (c) 2004 Alexander Djourik. All rights reserved.
 *
 */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Please see the file COPYING in this directory for full copyright
 * information.
 */

#ifndef FOO_TTA_H_
#define FOO_TTA_H_

#include "../SDK/foobar2000.h"
#include <mmreg.h>
#include "ttadec.h"
#include "resource.h"

#define  PLUGIN_VERSION	"2.4.1"
#define  COPYRIGHT		"Copyright (c) 2004 Alexander Djourik. All rights reserved."
#define  PROJECT_URL	"http://www.true-audio.com"

class input_tta {
	service_ptr_t<file> infile;
	bool seekable;

	BYTE iso_buffer[BUFFER_SIZE+4];	// Reader cache
	BYTE pcm_buffer[BUFFER_SIZE];	// PCM buffer
	decoder tta[2*MAX_NCH];			// decoder state
	long	cache[MAX_NCH];			// decoder cache
	volatile long bitrate;			// dynamic bitrate value

	tta_info info;					// currently playing file info

	unsigned long fframes;			// number of frames in file
	unsigned long framelen;			// the frame length in samples
	unsigned long lastlen;			// the length of the last frame in samples
	unsigned long data_pos;			// currently playing frame index
	unsigned long data_cur;			// the playing position in frame
	unsigned long seek_skip;        // number of samples to skip after seek
	unsigned long data_float;		// data type flag
	long maxvalue;					// output data max value

	unsigned long *seek_table;		// the playing position table
	unsigned long st_state;			// seek table status

	unsigned long frame_crc32;
	unsigned long bit_count;
	unsigned long bit_cache;
	unsigned char *bitpos;
	unsigned open_flags;

public:
	input_tta () {
		open_flags = 0;
		seek_table = NULL, st_state = 0;
		seek_skip = 0;
	}
	~input_tta () {
		if (open_flags & input_open_decode)
			if (seek_table) delete [] seek_table;
		//if (infile) infile->reader_release();
	}

	void open(service_ptr_t<file> p_filehint,const char * p_path,t_input_open_reason p_reason,abort_callback & p_abort);
	void get_info(file_info & p_info,abort_callback & p_abort);
	t_filestats get_file_stats(abort_callback & p_abort);
	void decode_initialize(unsigned p_flags,abort_callback & p_abort);
	bool decode_run(audio_chunk & p_chunk,abort_callback & p_abort);
	void decode_seek(double p_seconds,abort_callback & p_abort);
	bool decode_can_seek();
	bool decode_get_dynamic_info(file_info & p_out, double & p_timestamp_delta);
	bool decode_get_dynamic_info_track(file_info & p_out, double & p_timestamp_delta);
	void decode_on_idle(abort_callback & p_abort);
	void retag(const file_info & p_info,abort_callback & p_abort);
	static bool g_is_our_content_type(const char * p_content_type);
	static bool g_is_our_path(const char * p_path,const char * p_extension);

private:
	int open_tta_file(tta_info *info, abort_callback & p_abort);
	void tta_error (int error);
	void init_buffer_read(void);
	void get_binary(unsigned long *value, unsigned long bits, abort_callback & p_abort);
	void get_unary(unsigned long *value, abort_callback & p_abort);
	int done_buffer_read(abort_callback & p_abort);
	void decoder_init(decoder *tta, long nch, long byte_size);
	int set_position (unsigned long pos, abort_callback & p_abort);
	int player_init (abort_callback & p_abort);
	int get_samples (byte *buffer, long count, abort_callback & p_abort);
};

#endif /* FOO_TTA_H_ */
