#include "../SDK/foobar2000.h"
#include "tta/ttacodec.h"
#include "foo_input_tta.h"

class packet_decoder_tta : public packet_decoder
{
public:
    packet_decoder_tta()
    {
        ttacodec = NULL;
		pcmbuffer = NULL;
		pcmbuffer_size = 0;
    }

    ~packet_decoder_tta()
    {
		cleanup();
    }

	virtual t_size set_stream_property(const GUID & p_type,t_size p_param1,const void * p_param2,t_size p_param2size)
	{
		return 0;
	}

	virtual void get_info(file_info & p_info)
	{
		p_info.info_set_int("samplerate", ttacodec->srate);
		p_info.info_set_int("channels", ttacodec->num_chan);
		p_info.info_set_int("bitspersample", ttacodec->bps);
		p_info.info_set("codec", "True Audio");
		p_info.info_set("encoding", "lossless");
		p_info.info_set("homepage", PROJECT_URL);
	}

	virtual unsigned get_max_frame_dependency() { return 0; }
	virtual double get_max_frame_dependency_time() { return 0; }

	virtual void reset_after_seek()
	{
	}

	virtual void decode(const void * p_buffer,t_size p_bytes,audio_chunk & p_chunk,abort_callback & p_abort)
	{	
		ZeroMemory(pcmbuffer,pcmbuffer_size);
		unsigned long len = tta_codec_decoder_decompress(ttacodec,
			(unsigned char*)p_buffer, p_bytes, pcmbuffer, pcmbuffer_size);
		
		if(!len && p_bytes) throw exception_io_unsupported_format();

		if (ttacodec->is_float)
		{
			p_chunk.set_data_32((const float *)ttacodec->data,
				len / ttacodec->num_chan / ttacodec->byte_size,
				ttacodec->num_chan, ttacodec->srate);
		}
		else
		{
			p_chunk.set_data_fixedpoint(pcmbuffer, len, ttacodec->srate,
				ttacodec->num_chan, ttacodec->bps, audio_chunk::g_guess_channel_config(ttacodec->num_chan));
		}
	}

	virtual bool analyze_first_frame_supported() { return false; }

	virtual void analyze_first_frame(const void * p_buffer,t_size p_bytes,abort_callback & p_abort) {
	}

	void open(const GUID & p_owner,bool p_decode,t_size p_param1,const void * p_param2,t_size p_param2size,abort_callback & p_abort)
	{
		if (p_owner == owner_matroska)
		{
			if (p_param2size == sizeof(matroska_setup))
			{
				const matroska_setup * setup = (const matroska_setup*) p_param2;
				if (!strncmp(setup->codec_id,"A_TTA1",6))
				{
					cleanup();
					// !!!! Where is the bit per sample ???
					// well we just support 16 bits for now
					ttacodec = tta_codec_decoder_new(setup->sample_rate, setup->channels, 16);
					if(ttacodec)
					{
						pcmbuffer_size = tta_codec_get_frame_len(ttacodec->srate) *
							ttacodec->byte_size * ttacodec->num_chan;
						pcmbuffer = new unsigned char[pcmbuffer_size];
					}
					if (ttacodec == NULL) throw exception_io_data();
				}
				else throw exception_io_unsupported_format();
			}
			else throw exception_io_unsupported_format();
		}
		else throw exception_io_unsupported_format();
	}

	static bool g_is_our_setup(const GUID & p_owner,t_size p_param1,const void * p_param2,t_size p_param2size) {
		if (p_owner == owner_matroska)
		{
			if (p_param2size == sizeof(matroska_setup))
			{
				const matroska_setup * setup = (const matroska_setup*) p_param2;
				if (!strncmp(setup->codec_id,"A_TTA1",6)) {
					return true;
				} else return false;
			}
			else return false;
		}
		else return false;
	}

private:
	void cleanup() 
	{
		if(ttacodec)
		{
			tta_codec_free(ttacodec);
			ttacodec = NULL;
		}
		if(pcmbuffer)
		{
			delete[] pcmbuffer;
			pcmbuffer = NULL;
		}
		pcmbuffer_size = 0;
	}
	TTA_codec* ttacodec;
	unsigned char* pcmbuffer;
	unsigned long pcmbuffer_size;
};

static packet_decoder_factory_t<packet_decoder_tta> g_packet_decorder_tta_factory;
